@@ -1,18 +1,18 @@
 # Orca
 #
-# Copyright 2004-2008 Sun Microsystems Inc.
+# Copyright 2004-2009 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Library General Public
+# modify it under the terms of the GNU Lesser General Public
 # License as published by the Free Software Foundation; either
-# version 2 of the License, or (at your option) any later version.
+# version 2.1 of the License, or (at your option) any later version.
 #
 # This library is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Library General Public License for more details.
+# Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU Library General Public
+# You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the
 # Free Software Foundation, Inc., Franklin Street, Fifth Floor,
 # Boston MA  02110-1301 USA.
@@ -23,7 +23,7 @@ as its speech server, or it can feel free to create one of its own."""
 __id__        = "$Id$"
 __version__   = "$Revision$"
 __date__      = "$Date$"
-__copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
+__copyright__ = "Copyright (c) 2005-2009 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import logging
@@ -38,6 +38,7 @@ import keynames
 import orca
 import orca_state
 import settings
+import sound
 import speech_generator
 
 from acss import ACSS
@@ -70,31 +71,19 @@ def getSpeechServerFactories():
                                  [''])
             factories.append(module)
         except:
-            debug.printException(debug.LEVEL_OFF)
+            debug.printException(debug.LEVEL_CONFIGURATION)
 
     return factories
 
-def init():
+def _initSpeechServer(moduleName, speechServerInfo):
 
     global _speechserver
 
-    if _speechserver:
+    if not moduleName:
         return
 
-    # First, find the factory module to use.  We will
-    # allow the user to give their own factory module,
-    # thus we look first in the global name space, and
-    # then we look in the orca namespace.
-    #
-    moduleName = settings.speechServerFactory
-
-    if moduleName:
-        debug.println(debug.LEVEL_CONFIGURATION,
-                      "Using speech server factory: %s" % moduleName)
-    else:
-        debug.println(debug.LEVEL_CONFIGURATION,
-                      "Speech not available.")
-        return
+    debug.println(debug.LEVEL_CONFIGURATION,
+                  "Trying to use speech server factory: %s" % moduleName)
 
     factory = None
     try:
@@ -117,8 +106,41 @@ def init():
     speechServerInfo = settings.speechServerInfo
     if speechServerInfo:
         _speechserver = factory.SpeechServer.getSpeechServer(speechServerInfo)
-    else:
+
+    if not _speechserver:
         _speechserver = factory.SpeechServer.getSpeechServer()
+        if speechServerInfo:
+            debug.println(debug.LEVEL_CONFIGURATION,
+                          "Invalid speechServerInfo: %s" % speechServerInfo)
+
+    if not _speechserver:
+        raise Exception("No speech server for factory: %s" % moduleName)
+
+def init():
+
+    if _speechserver:
+        return
+
+    try:
+        moduleName = settings.speechServerFactory
+        _initSpeechServer(moduleName,
+                          settings.speechServerInfo)
+    except:
+        moduleNames = settings.speechFactoryModules
+        for moduleName in moduleNames:
+            if moduleName != settings.speechServerFactory:
+                try:
+                    _initSpeechServer(moduleName, None)
+                    if _speechserver:
+                        break
+                except:
+                    debug.printException(debug.LEVEL_SEVERE)
+
+    if _speechserver:
+        debug.println(debug.LEVEL_CONFIGURATION,
+                      "Using speech server factory: %s" % moduleName)
+    else:
+        debug.println(debug.LEVEL_CONFIGURATION, "Speech not available.")
 
 def __resolveACSS(acss=None):
     if acss:
@@ -144,7 +166,7 @@ def _speak(text, acss, interrupt):
     if settings.speakMultiCaseStringsAsWords:
         text = _processMultiCaseString(text)
     if orca_state.activeScript and orca_state.usePronunciationDictionary:
-        text = orca_state.activeScript.adjustForPronunciation(text)
+        text = orca_state.activeScript.utilities.adjustForPronunciation(text)
     if settings.speakMultiCaseStringsAsWords:
         text = _processMultiCaseString(text)
 
@@ -160,62 +182,78 @@ def _speak(text, acss, interrupt):
     log.info(logLine + extraDebug)
 
     if _speechserver:
-        _speechserver.speak(text, __resolveACSS(acss), interrupt)
-
+        voice = ACSS(settings.voices.get(settings.DEFAULT_VOICE))
+        try:
+            voice.update(acss)
+        except:
+            pass
+        _speechserver.speak(text, __resolveACSS(voice), interrupt)
 
 def speak(content, acss=None, interrupt=True):
     """Speaks the given content.  The content can be either a simple
     string or an array of arrays of objects returned by a speech
     generator."""
 
+    if settings.silenceSpeech:
+        return
+
+    validTypes = (basestring, list, sound.Sound, speech_generator.Pause,
+                  speech_generator.LineBreak, ACSS)
+    error = "bad content sent to speech.speak: '%s'"
+    if not isinstance(content, validTypes):
+        debug.printStack(debug.LEVEL_WARNING)
+        debug.println(debug.LEVEL_WARNING, error % content)
+        return
+
     # We will not interrupt a key echo in progress.
     #
     if orca_state.lastKeyEchoTime:
         interrupt = interrupt \
             and ((time.time() - orca_state.lastKeyEchoTime) > 0.5)
 
-    if settings.silenceSpeech:
+    if isinstance(content, basestring):
+        _speak(content, acss, interrupt)
+    elif isinstance(content, sound.Sound):
+        content.play()
+    if not isinstance(content, list):
         return
 
-    if isinstance(content, basestring):
-        subString = content
-    elif isinstance(content, list):
-        subString = None
-        for element in content:
-            if isinstance(element, basestring):
-                if subString:
-                    subString += " " + element
-                else:
-                    subString = element
-            elif isinstance(element, speech_generator.Pause):
-                if subString:
-                    if subString[-1] != ".":
-                        subString += "."
-                    _speak(subString, acss, interrupt)
-                    subString = None
-            elif isinstance(element, speech_generator.LineBreak):
-                if subString:
-                    _speak(subString, acss, interrupt)
-                    subString = None
-            else:
-                if subString:
-                    _speak(subString, acss, interrupt)
-                subString = None
-                if isinstance(element, list):
-                    speak(element, acss, interrupt)
-                elif isinstance(element, ACSS):
-                    acss = ACSS(acss)
-                    acss.update(element)
-                else:
-                    debug.println(debug.LEVEL_WARNING,
-                                  "UNKNOWN speech element: '%s'" % element)
-    else:
-        debug.printStack(debug.LEVEL_WARNING)
-        debug.println(debug.LEVEL_WARNING, 
-                      "bad content send to speech.speak: '%s'", repr(content))
+    toSpeak = []
+    activeVoice = ACSS(acss)
+    for element in content:
+        if not isinstance(element, validTypes):
+            debug.println(debug.LEVEL_WARNING, error % element)
+        elif isinstance(element, list):
+            speak(element, acss, interrupt)
+        elif isinstance(element, basestring):
+            if len(element):
+                toSpeak.append(element)
+        elif toSpeak:
+            newVoice = ACSS(acss)
+            newItemsToSpeak = []
+            if isinstance(element, speech_generator.Pause):
+                if not toSpeak[-1].endswith('.'):
+                    toSpeak[-1] += '.'
+                if not settings.enablePauseBreaks:
+                    continue
+            elif isinstance(element, ACSS):
+                newVoice.update(element)
+                if newVoice == activeVoice:
+                    continue
+                newItemsToSpeak.append(toSpeak.pop())
 
-    if subString:
-        _speak(subString, acss, interrupt)
+            if toSpeak:
+                string = " ".join(toSpeak)
+                _speak(string, activeVoice, interrupt)
+            activeVoice = newVoice
+            toSpeak = newItemsToSpeak
+
+        if isinstance(element, sound.Sound):
+            element.play()
+
+    if toSpeak:
+        string = " ".join(toSpeak)
+        _speak(string, activeVoice, interrupt)
 
 def speakKeyEvent(event_string, eventType):
     """Speaks a key event immediately.
@@ -291,6 +329,20 @@ def stop():
     if _speechserver:
         _speechserver.stop()
 
+
+
+def updatePunctuationLevel(script=None, inputEvent=None):
+    """ Punctuation level changed, inform this speechServer. """
+
+    if _speechserver:
+        _speechserver.updatePunctuationLevel()
+    else:
+        logLine = "SPEECH OUTPUT: 'punctuation level' updated"
+        debug.println(debug.LEVEL_INFO, logLine)
+        log.info(logLine)
+
+    return True
+
 def increaseSpeechRate(script=None, inputEvent=None):
     if _speechserver:
         _speechserver.increaseSpeechRate()
